/*--------------------------------------------------------------------
 * FICHERO:  SalvaPan.c
 * OBJETIVO: Manejar la parte especfica del salvapantallas
 * AUTOR:    Pedro Reina
 * FECHA:    X.24.2.1999
 *------------------------------------------------------------------*/

/*--------------------------------------------------------------------
 * Ficheros de cabecera
 *------------------------------------------------------------------*/

#include <stdlib.h>   // rand()
#include "SpPedro.h"

/*--------------------------------------------------------------------
 * Declaracin de funciones
 *------------------------------------------------------------------*/

void PreparaLista (int *, int);

/*--------------------------------------------------------------------
 * Definicin de macros constantes
 *------------------------------------------------------------------*/

#define ID_CRONO  301
#define LADO      20

/*--------------------------------------------------------------------
 * Definicin de macros funciones
 *------------------------------------------------------------------*/

/*--------------------------------------------------------------------
 * MACRO:    Azr_Entero()
 * OBJETIVO: Obtener un nmero entero aleatorio en un rango especificado
 * ENTRADAS: Los valores ms bajo y ms alto admitidos
 * SALIDAS:  Un nmero entero entre los extremos (ambos inclusive)
 *------------------------------------------------------------------*/
#define Azr_Entero(a,b)  ((a)+(rand()%((b)-(a)+1)))

/*--------------------------------------------------------------------
 * Definicin de variables globales
 *------------------------------------------------------------------*/

// Ninguna

/*--------------------------------------------------------------------
 * Definicin de funciones
 *------------------------------------------------------------------*/

/*--------------------------------------------------------------------
 * FUNCION:  SalvaPantallas()
 * OBJETIVO: Activar el salvapantallas
 * ENTRADAS: El modo de mostrar la ventana
 * SALIDAS:  El ltimo mensaje
 *------------------------------------------------------------------*/
int
SalvaPantallas
  (
  int ModoMostrar
  )
  {
  HWND  VentanaTotal, Anterior;
  MSG   Mensaje;
  HMENU MenuControl;

  // Creamos la ventana que ocupa toda la pantalla
  VentanaTotal = CreateWindow (
    "VenTotal",      //  Clase de ventana
    NULL,            //  Ttulo
    WS_POPUP,        //  Estilo de ventana
    0,               //  X esquina superior izquierda
    0,               //  Y esquina superior izquierda
    AnchoPantalla(), //  Anchura
    AltoPantalla(),  //  Altura
    (HWND)NULL,      //  Ventana padre
    (HMENU)NULL,     //  Men o ventana hija
    Aplic,           //  Instancia de aplicacin
    (LPSTR)NULL      //  Puntero a valor...
    );

  // Almacenamos la ventana activa
  Anterior = GetActiveWindow();

  // Mostramos la ventana
  ShowWindow (VentanaTotal, ModoMostrar);

  // Manejamos los mensajes
  while ( GetMessage (&Mensaje, NULL, NULL, NULL) )
    {
    TranslateMessage (&Mensaje);
    DispatchMessage (&Mensaje);
    }

  // Devolvemos la ventana activa
  SetActiveWindow (Anterior);

  return ( Mensaje.wParam );
  }

/*--------------------------------------------------------------------
 * FUNCION:  ProcedimientoSalvaPantallas()
 * OBJETIVO: Recibir los mensajes de Windows para la ventana
 *           del salvapantallas
 *------------------------------------------------------------------*/
#pragma argsused
LONG FAR PASCAL
ProcedimientoSalvaPantallas
  (
  HWND   Ventana,        // Ventana a la que se enva el mensaje
  UINT   Mensaje,        // El mensaje
  WPARAM ParametroCorto, // Informacin adicional
  LPARAM ParametroLargo  // Informacin adicional
  )
  {
  BOOL        Respuesta = FALSE;
  UINT        Cronometro;
  PAINTSTRUCT Pintura;
  HDC         Contexto;
  static RECT Rectangulo, Pantalla;
  static int *Lista, Total, Anchura, Fila, Columna, i;
  int         Altura, Color;
  static BOOL Borrando = TRUE;

  switch ( Mensaje )
    {
    case WM_CREATE:
      // Eliminamos el puntero
      ShowCursor (FALSE);

      // Creamos un cronmetro
      Cronometro = SetTimer (Ventana, ID_CRONO, 5, NULL);

      // Definimos la pantalla completa
      Pantalla.left   = 0;
      Pantalla.top    = 0;
      Pantalla.right  = AnchoPantalla();
      Pantalla.bottom = AltoPantalla();

      // Valores iniciales
      Anchura = AnchoPantalla() / LADO + (AnchoPantalla()%LADO>0);
      Altura = AltoPantalla() / LADO + (AltoPantalla()%LADO>0);
      Total = Anchura*Altura;
      i = 0;
      
      // Preparamos la lista con los cuadritos
      Lista = (int *) LocalAlloc (LMEM_FIXED, Total*sizeof(int));
      PreparaLista (Lista, Total);
      break;

    case WM_KEYDOWN:
    case WM_SYSKEYDOWN:
    case WM_RBUTTONDOWN:
    case WM_LBUTTONDOWN:
      SendMessage (Ventana, WM_CLOSE, 0, 0L);
      break;

    case WM_SYSCOMMAND:
      if ( ParametroCorto == SC_SCREENSAVE )
        { Respuesta = TRUE; }
      break;

    case WM_TIMER:
      Fila = Lista[i] / Anchura;
      Columna = Lista[i] - Anchura * Fila;
      Rectangulo.left   = LADO*Columna;
      Rectangulo.top    = LADO*Fila;
      Rectangulo.right  = Rectangulo.left + LADO;
      Rectangulo.bottom = Rectangulo.top + LADO;
      i++;
      if ( i == Total )
        {
        i = 0;
        if ( Borrando )  // Ya hemos borrado la pantalla
          {
          Borrando = FALSE;
          
          // Cambiamos el cronmetro
          if ( Cronometro == 0)  { KillTimer (Ventana, ID_CRONO); }
          Cronometro = SetTimer (Ventana, ID_CRONO, 500, NULL);
          }
        }
      if ( Borrando )
        { InvalidateRect (Ventana, &Rectangulo, TRUE); }
      else
        { InvalidateRect (Ventana, &Pantalla, TRUE); }
      UpdateWindow (Ventana);
      break;

    case WM_PAINT:
      Contexto = BeginPaint (Ventana, &Pintura);
      if ( !Borrando )
        { FillRect (Contexto, &Pantalla, GetStockObject(BLACK_BRUSH)); }
      Color = Borrando ? BLACK_BRUSH : LTGRAY_BRUSH;
      FillRect (Contexto, &Rectangulo, GetStockObject(Color));
      EndPaint (Ventana, &Pintura);
      break;

    case WM_CLOSE:
      // Recuperamos el puntero
      ShowCursor (TRUE);
      SendMessage (Ventana, WM_DESTROY, 0, 0L);
      break;

    case WM_DESTROY:
      // Destruimos el cronmetro
      if ( Cronometro == 0)  { KillTimer (Ventana, ID_CRONO); }

      // Liberamos memoria
      LocalFree ((HLOCAL)Lista);

      PostQuitMessage (0);
      break;

    default:
      Respuesta = DefWindowProc (Ventana, Mensaje,
                                 ParametroCorto, ParametroLargo);
      break;
    }

  return ( Respuesta );
  }

/*--------------------------------------------------------------------
 * FUNCION:  PreparaLista()
 * OBJETIVO: Preparar una lista con los N primeros nmeros
 *           colocados aleatoriamente
 * ENTRADAS: La lista y N
 * SALIDAS:  Ninguna
 *------------------------------------------------------------------*/
void
PreparaLista
  (
  int *Lista,
  int  N
  )
  {
  int i, j, Tmp;

  // Rellenamos la lista con nmeros consecutivos
  for ( i=0 ; i<N ; i++ )
    { Lista[i] = i; }

  // Los mezclamos
  for ( i=0 ; i<N ; i++ )
    {
    j = Azr_Entero (0, N-1);
    Tmp = Lista[i];
    Lista[i] = Lista[j];
    Lista[j] = Tmp;
    }
  }