/*--------------------------------------------------------------------
 * FICHERO:  SpPedro.c
 * OBJETIVO: Un sencillo salvapantallas para MS Windows 3.1
 * AUTOR:    Pedro Reina
 * FECHA:    L.18.5.1998
 *------------------------------------------------------------------*/

/*--------------------------------------------------------------------
 * Ficheros de cabecera
 *------------------------------------------------------------------*/

#include <Windows.h>
#include "SpPedro.h"

/*--------------------------------------------------------------------
 * Definicin de tipos
 *------------------------------------------------------------------*/

// Ninguno

/*--------------------------------------------------------------------
 * Declaracin de funciones
 *------------------------------------------------------------------*/

LONG FAR PASCAL ProcedimientoPrincipal (HWND, UINT, WPARAM, LPARAM);
BOOL FAR PASCAL ProcedimientoAcercaDe (HWND,UINT,WPARAM,LPARAM);
BOOL RegistraClasesVentana (void);
int  Presentacion (int);
void BotonNormal (HWND,int);
void BotonDefecto (HWND,int);

/*--------------------------------------------------------------------
 * Definicin de macros constantes
 *------------------------------------------------------------------*/

// Ninguno

/*--------------------------------------------------------------------
 * Definicin de macros funciones
 *------------------------------------------------------------------*/

// Ninguno

/*--------------------------------------------------------------------
 * Definicin de variables globales
 *------------------------------------------------------------------*/

FARPROC PunteroAcercaDe;
FARPROC PunteroAyuda;
HANDLE  Aplic;

/*--------------------------------------------------------------------
 * Programa principal
 *------------------------------------------------------------------*/
#pragma argsused
int PASCAL
WinMain
  (
  HANDLE Instancia,    // Manejador de instancia de aplicacin
  HANDLE Anterior,     // Manejador de instancia anterior
  LPSTR  Parametros,   // Cadena con la lnea de parmetros
  int    ModoMostrar   // Aplicacin como ventana o como icono
  )
  {
  int Respuesta;

  // Almacenamos la Instancia en una variable global
  Aplic = Instancia;

  // Si es necesario, registramos las clases de ventana
  if ( !Anterior )
    {
    if ( !RegistraClasesVentana() )
      { return ( FALSE ); }
    }

  // Lanzamos el salvapantallas o la presentacin
  if ( lstrcmp (Parametros,"/s") == 0 )
    { Respuesta = SalvaPantallas (ModoMostrar); }
  else
    { Respuesta = Presentacion (ModoMostrar); }

  // Y salimos
  return ( Respuesta );
  }

/*--------------------------------------------------------------------
 * Definicin de funciones
 *------------------------------------------------------------------*/

/*--------------------------------------------------------------------
 * FUNCION:  RegistraClasesVentana()
 * OBJETIVO: Registrar las dos clases de ventana que necesita
 *           este programa
 * ENTRADAS: Ninguna
 * SALIDAS:  Lgica, que indica si se han podido registrar las clases
 *------------------------------------------------------------------*/
BOOL
RegistraClasesVentana (void)
  {
  BOOL     Respuesta = TRUE;
  WNDCLASS Clase;

  // Comenzamos con la ventana del cuadro de dilogo
  // de presentacin

  // Estilo de ventana
  Clase.style = CS_HREDRAW | CS_VREDRAW;

  // Funcin que maneja la ventana
  Clase.lpfnWndProc = ProcedimientoPrincipal;

  // Octetos extra
  Clase.cbClsExtra = 0;

  // Octetos extra
  Clase.cbWndExtra = DLGWINDOWEXTRA;

  // Instancia de aplicacin
  Clase.hInstance = Aplic;

  // Icono al minimizar
  Clase.hIcon = LoadIcon (Aplic,"SpPedro");

  // Cursor
  Clase.hCursor = LoadCursor (NULL,IDC_ARROW);

  // Pincel de fondo
  Clase.hbrBackground = COLOR_WINDOW + 1;

  // Recurso de men
  Clase.lpszMenuName = NULL;

  // Nombre de la clase de ventana
  Clase.lpszClassName = "SpPedro";

  if ( !RegisterClass (&Clase) )
    { Respuesta = FALSE; }

  // Y seguimos con la ventana que ocupar toda la pantalla

  if ( Respuesta )
    {
    // Estilo de ventana
    Clase.style = CS_VREDRAW | CS_HREDRAW | CS_SAVEBITS | CS_DBLCLKS;

    // Funcin que maneja la ventana
    Clase.lpfnWndProc = ProcedimientoSalvaPantallas;

    // Octetos extra
    Clase.cbClsExtra = 0;

    // Octetos extra
    Clase.cbWndExtra = 0;

    // Instancia de aplicacin
    Clase.hInstance = Aplic;

    // Icono al minimizar
    Clase.hIcon = LoadIcon (Aplic,"SpPedro");

    // Cursor
    Clase.hCursor = NULL;

    // Pincel de fondo
    Clase.hbrBackground = NULL;

    // Recurso de men
    Clase.lpszMenuName = NULL;

    // Nombre de la clase de ventana
    Clase.lpszClassName = "VenTotal";

    if ( !RegisterClass (&Clase) )
      { Respuesta = FALSE; }
    }

  return ( Respuesta );
  }

/*--------------------------------------------------------------------
 * FUNCION:  Presentacion()
 * OBJETIVO: Presentar el salvapantallas y llevar a las opciones
 *           de informacin
 * ENTRADAS: El modo de mostrar la ventana
 * SALIDAS:  El ltimo mensaje
 *------------------------------------------------------------------*/
int
Presentacion
  (
  int ModoMostrar
  )
  {
  HWND  VentanaPrincipal;
  MSG   Mensaje;
  HMENU MenuControl;

  // Creamos la ventana principal
  VentanaPrincipal = CreateDialog (Aplic, "Principal", 0, NULL);

  // Cambiamos el men del cuadro de control
  MenuControl = GetSystemMenu (VentanaPrincipal, FALSE);
  DeleteMenu (MenuControl, SC_SIZE,     MF_BYCOMMAND);
  DeleteMenu (MenuControl, SC_MAXIMIZE, MF_BYCOMMAND);

  // Preparamos las llamadas a los cuadros de dilogo
  PunteroAcercaDe = MakeProcInstance (ProcedimientoAcercaDe, Aplic);
  PunteroAyuda = MakeProcInstance (ProcedimientoAyuda, Aplic);

  // Mostramos la ventana
  CentraVentana (VentanaPrincipal);
  ShowWindow (VentanaPrincipal, ModoMostrar);

  // Manejamos los mensajes
  while ( GetMessage (&Mensaje, NULL, NULL, NULL) )
    {
    TranslateMessage (&Mensaje);
    DispatchMessage (&Mensaje);
    }

  return ( Mensaje.wParam );
  }

/*--------------------------------------------------------------------
 * FUNCION:  ProcedimientoPrincipal()
 * OBJETIVO: Recibir los mensajes de Windows para la clase
 *           de ventanas principal de esta aplicacin
 *------------------------------------------------------------------*/
LONG FAR PASCAL
ProcedimientoPrincipal
  (
  HWND   Ventana,        // Ventana a la que se enva el mensaje
  UINT   Mensaje,        // El mensaje
  WPARAM ParametroCorto, // Informacin adicional
  LPARAM ParametroLargo  // Informacin adicional
  )
  {
  static int Opcion = ID_ACERCADE;
  BOOL       Respuesta = FALSE;

  switch ( Mensaje )
    {
    case WM_KEYDOWN:
      BotonNormal (Ventana, Opcion);
      switch ( ParametroCorto )
        {
        case VK_TAB:
          if ( GetKeyState (VK_SHIFT) < 0 )  { Opcion--; }
          else                               { Opcion++; }
          break;
        case VK_RIGHT: Opcion++; break;
        case VK_LEFT:  Opcion--; break;
        case VK_SPACE:
        case VK_RETURN:
          SendMessage (Ventana, WM_COMMAND, Opcion, 0L);
          break;
        case VK_ESCAPE:  // Sigue a la siguiente opcin (Salir)
        case 'S': SendMessage (Ventana, WM_COMMAND, ID_SALIR,    0L); break;
        case 'A': SendMessage (Ventana, WM_COMMAND, ID_ACERCADE, 0L); break;
        case 'T': SendMessage (Ventana, WM_COMMAND, ID_AUTOR,    0L); break;
        }
      if ( Opcion > ID_AUTOR )  { Opcion = ID_SALIR; }
      if ( Opcion < ID_SALIR )  { Opcion = ID_AUTOR; }
      BotonDefecto (Ventana, Opcion);
      break;

    case WM_COMMAND:
      switch ( ParametroCorto )
        {
        case ID_SALIR:
          SendMessage (Ventana, WM_CLOSE, 0, 0L);
          Respuesta = TRUE;
          break;

        case ID_ACERCADE:
          DialogBox (Aplic, "AcercaDe", Ventana, PunteroAcercaDe);
          Respuesta = TRUE;
          break;

        case ID_AUTOR:
          MuestraAyuda (Ventana, "Autor", "AyudaAutor");
          break;
        }
      break;

    case WM_CLOSE:
        SendMessage (Ventana, WM_DESTROY, 0, 0L);
      break;

    case WM_DESTROY: PostQuitMessage (0);  break;

    default:
      Respuesta = DefWindowProc (Ventana, Mensaje,
                                 ParametroCorto, ParametroLargo);
      break;
    }

  return ( Respuesta );
  }

/*--------------------------------------------------------------------
 * FUNCION:  BotonNormal()
 * OBJETIVO: Poner un boton de un cuadro de dilogo como normal
 * ENTRADAS: La ventana del cuadro
 * SALIDAS:  Ninguna
 *------------------------------------------------------------------*/
void
BotonNormal
  (
  HWND Ventana,
  int  Opcion
  )
  {
  HWND Boton;

  Boton = GetDlgItem (Ventana, Opcion);
  SetWindowLong (Boton, GWL_STYLE, BS_PUSHBUTTON | WS_CHILD | WS_VISIBLE);
  InvalidateRect (Boton, NULL, TRUE);
  UpdateWindow (Boton);
  }

/*--------------------------------------------------------------------
 * FUNCION:  BotonDefecto()
 * OBJETIVO: Poner un boton de un cuadro de dilogo como botn por defecto
 * ENTRADAS: La ventana del cuadro
 * SALIDAS:  Ninguna
 *------------------------------------------------------------------*/
void
BotonDefecto
  (
  HWND Ventana,
  int  Opcion
  )
  {
  HWND Boton;

  Boton = GetDlgItem (Ventana, Opcion);
  SetWindowLong (Boton, GWL_STYLE, BS_DEFPUSHBUTTON | WS_CHILD | WS_VISIBLE);
  InvalidateRect (Boton, NULL, TRUE);
  UpdateWindow (Boton);
  }

/*--------------------------------------------------------------------
 * FUNCION:  CentraVentana()
 * OBJETIVO: Centrar una ventana en pantalla
 * ENTRADAS: La ventana
 * SALIDAS:  Lgica, que indica si ha cambiado la posicin de la ventana
 * NOTA:     Slo se cambia la posicin
 *------------------------------------------------------------------*/
BOOL
CentraVentana
  (
  HWND Ventana
  )
  {
  BOOL Respuesta = FALSE;
  RECT Info;
  int  AnchoVentana, AltoVentana, Izquierda, Arriba;

  GetWindowRect (Ventana, &Info);

  AnchoVentana = Info.right - Info.left;
  AltoVentana = Info.bottom - Info.top;

  Izquierda = (AnchoPantalla()-AnchoVentana) / 2;
  Arriba = (AltoPantalla()-AltoVentana) / 2;

  if ( Izquierda >= 0 && Arriba >= 0 )
    {
    SetWindowPos (Ventana, HWND_TOP, Izquierda, Arriba, 0, 0,
                  SWP_NOACTIVATE | SWP_NOSIZE | SWP_NOREDRAW | SWP_NOZORDER);
    Respuesta = TRUE;
    }

  return ( Respuesta );
  }
